@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 Igalia S.L.
+ * Copyright (C) 2010, 2011 Igalia S.L.
  *
  * Contact: Iago Toral Quiroga <itoral@igalia.com>
  *
@@ -38,30 +38,15 @@
 
 #include "grl-data.h"
 #include "grl-log.h"
-#include <grl-plugin-registry.h>
+#include <grl-registry.h>
 
 #define GRL_LOG_DOMAIN_DEFAULT data_log_domain
 GRL_LOG_DOMAIN(data_log_domain);
 
-enum {
-  PROP_0,
-  PROP_OVERWRITE
-};
-
 struct _GrlDataPrivate {
   GHashTable *data;
 };
 
-static void grl_data_set_property (GObject *object,
-                                   guint prop_id,
-                                   const GValue *value,
-                                           GParamSpec *pspec);
-
-static void grl_data_get_property (GObject *object,
-                                   guint prop_id,
-                                   GValue *value,
-                                   GParamSpec *pspec);
-
 static void grl_data_finalize (GObject *object);
 static void free_list_values (GrlKeyID key, GList *values, gpointer user_data);
 
@@ -79,19 +64,9 @@ grl_data_class_init (GrlDataClass *klass)
 {
   GObjectClass *gobject_class = (GObjectClass *)klass;
 
-  gobject_class->set_property = grl_data_set_property;
-  gobject_class->get_property = grl_data_get_property;
   gobject_class->finalize = grl_data_finalize;
 
   g_type_class_add_private (klass, sizeof (GrlDataPrivate));
-
-  g_object_class_install_property (gobject_class,
-                                   PROP_OVERWRITE,
-                                   g_param_spec_boolean ("overwrite",
-                                                         "Overwrite",
-                                                         "Overwrite current values",
-                                                         TRUE,
-                                                         G_PARAM_READWRITE));
 }
 
 static void
@@ -118,40 +93,6 @@ grl_data_finalize (GObject *object)
   G_OBJECT_CLASS (grl_data_parent_class)->finalize (object);
 }
 
-static void
-grl_data_set_property (GObject *object,
-                       guint prop_id,
-                       const GValue *value,
-                       GParamSpec *pspec)
-{
-  switch (prop_id) {
-  case PROP_OVERWRITE:
-    g_warning ("\"overwrite\" property is deprecated");
-
-  default:
-    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    break;
-  }
-}
-
-static void
-grl_data_get_property (GObject *object,
-                       guint prop_id,
-                       GValue *value,
-                       GParamSpec *pspec)
-{
-  switch (prop_id) {
-  case PROP_OVERWRITE:
-    g_warning ("\"overwrite\" property is deprecated");
-    g_value_set_boolean (value, TRUE);
-    break;
-
-  default:
-    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    break;
-  }
-}
-
 /* ================ Utitilies ================ */
 
 /* Free the list of values, which are of type #GrlRelatedKeys */
@@ -165,19 +106,19 @@ free_list_values (GrlKeyID key, GList *values, gpointer user_data)
 static GrlKeyID
 get_sample_key (GrlKeyID key)
 {
-  GrlPluginRegistry *registry;
+  GrlRegistry *registry;
   const GList *related_keys;
 
-  registry = grl_plugin_registry_get_default ();
+  registry = grl_registry_get_default ();
   related_keys =
-    grl_plugin_registry_lookup_metadata_key_relation (registry, key);
+    grl_registry_lookup_metadata_key_relation (registry, key);
 
   if (!related_keys) {
     GRL_WARNING ("Related keys not found for key \"%s\"",
                  grl_metadata_key_get_name (key));
-    return NULL;
+    return GRL_METADATA_KEY_INVALID;
   } else {
-    return related_keys->data;
+    return GRLPOINTER_TO_KEYID (related_keys->data);
   }
 }
 
@@ -202,7 +143,7 @@ grl_data_new (void)
 /**
  * grl_data_get:
  * @data: data to retrieve value
- * @key: (type GObject.ParamSpec): key to look up.
+ * @key: (type GrlKeyID): key to look up.
  *
  * Get the first value from @data associated with @key.
  *
@@ -233,7 +174,7 @@ grl_data_get (GrlData *data, GrlKeyID key)
 /**
  * grl_data_set:
  * @data: data to modify
- * @key: (type GObject.ParamSpec): key to change or add
+ * @key: (type GrlKeyID): key to change or add
  * @value: the new value
  *
  * Sets the first value associated with @key in @data. If key already has a
@@ -276,7 +217,7 @@ grl_data_set (GrlData *data, GrlKeyID key, const GValue *value)
 /**
  * grl_data_set_string:
  * @data: data to modify
- * @key: (type GObject.ParamSpec): key to change or add
+ * @key: (type GrlKeyID): key to change or add
  * @strvalue: the new value
  *
  * Sets the first string value associated with @key in @data. If @key already
@@ -302,7 +243,7 @@ grl_data_set_string (GrlData *data,
 /**
  * grl_data_get_string:
  * @data: data to inspect
- * @key: (type GObject.ParamSpec): key to use
+ * @key: (type GrlKeyID): key to use
  *
  * Returns the first string value associated with @key from @data. If @key has
  * no first value, or value is not string, or @key is not in @data, then %NULL
@@ -328,7 +269,7 @@ grl_data_get_string (GrlData *data, GrlKeyID key)
 /**
  * grl_data_set_int:
  * @data: data to change
- * @key: (type GObject.ParamSpec): key to change or add
+ * @key: (type GrlKeyID): key to change or add
  * @intvalue: the new value
  *
  * Sets the first int value associated with @key in @data. If @key already has a
@@ -348,7 +289,7 @@ grl_data_set_int (GrlData *data, GrlKeyID key, gint intvalue)
 /**
  * grl_data_get_int:
  * @data: data to inspect
- * @key: (type GObject.ParamSpec): key to use
+ * @key: (type GrlKeyID): key to use
  *
  * Returns the first int value associated with @key from @data. If @key has no
  * first value, or value is not a gint, or @key is not in data, then 0 is
@@ -373,7 +314,7 @@ grl_data_get_int (GrlData *data, GrlKeyID key)
 /**
  * grl_data_set_float:
  * @data: data to change
- * @key: (type GObject.ParamSpec): key to change or add
+ * @key: (type GrlKeyID): key to change or add
  * @floatvalue: the new value
  *
  * Sets the first float value associated with @key in @data. If @key already has
@@ -393,7 +334,7 @@ grl_data_set_float (GrlData *data, GrlKeyID key, float floatvalue)
 /**
  * grl_data_get_float:
  * @data: data to inspect
- * @key: (type GObject.ParamSpec): key to use
+ * @key: (type GrlKeyID): key to use
  *
  * Returns the first float value associated with @key from @data. If @key has no
  * first value, or value is not a gfloat, or @key is not in data, then 0 is
@@ -418,12 +359,14 @@ grl_data_get_float (GrlData *data, GrlKeyID key)
 /**
  * grl_data_set_binary:
  * @data: data to change
- * @key: (type GObject.ParamSpec): key to change or add
+ * @key: (type GrlKeyID): key to change or add
  * @buf: buffer holding the data
  * @size: size of the buffer
  *
  * Sets the first binary value associated with @key in @data. If @key already
  * has a first value old value is replaced by the new one.
+ *
+ * Since: 0.1.9
  **/
 void
 grl_data_set_binary (GrlData *data, GrlKeyID key, const guint8 *buf, gsize size)
@@ -448,7 +391,7 @@ grl_data_set_binary (GrlData *data, GrlKeyID key, const guint8 *buf, gsize size)
 /**
  * grl_data_get_binary:
  * @data: data to inspect
- * @key: (type GObject.ParamSpec): key to use
+ * @key: (type GrlKeyID): key to use
  * @size: (out): location to store the buffer size
  *
  * Returns the first binary value associated with @key from @data. If @key has
@@ -457,6 +400,8 @@ grl_data_set_binary (GrlData *data, GrlKeyID key, const guint8 *buf, gsize size)
  *
  * Returns: buffer location associated with the @key, or %NULL in other case. If
  * successful @size will be set the to the buffer size.
+ *
+ * Since: 0.1.9
  **/
 const guint8 *
 grl_data_get_binary(GrlData *data, GrlKeyID key, gsize *size)
@@ -477,27 +422,60 @@ grl_data_get_binary(GrlData *data, GrlKeyID key, gsize *size)
 }
 
 /**
- * grl_data_add:
- * @data: data to change
- * @key: (type GObject.ParamSpec): key to add
+ * grl_data_set_boxed:
+ * @data: data to modify
+ * @key: key to change or add
+ * @boxed: the new value
  *
- * Adds a new @key to @data, with no value. If key already exists, it does
- * nothing.
+ * Sets the first boxed value associated with @key in @data. If @key already
+ * has a value, the old value is freed and the new one is set.
  *
- * Since: 0.1.4
- * Deprecated: 0.1.13: Use grl_data_add_related_keys() instead
+ * Since: 0.2.0
  **/
 void
-grl_data_add (GrlData *data, GrlKeyID key)
+grl_data_set_boxed (GrlData *data, GrlKeyID key, gconstpointer boxed)
 {
-  GRL_WARNING ("grl_data_add() is deprecated. Added keys require a value. "
-               "Use instead grl_data_set()");
+  GValue value = { 0 };
+
+  g_return_if_fail (boxed != NULL);
+
+  g_value_init (&value, GRL_METADATA_KEY_GET_TYPE (key));
+  g_value_set_boxed (&value, boxed);
+  grl_data_set (data, key, &value);
+  g_value_unset (&value);
+}
+
+/**
+ * grl_data_get_boxed:
+ * @data: data to inspect
+ * @key: (type GrlKeyID): key to use
+ *
+ * Returns the first boxed value associated with @key from @data. If @key has
+ * no first value, that value is not of a boxed type, or @key is not in @data,
+ * then %NULL is returned.
+ *
+ * Returns: (transfer none): the boxed instance associated with @key if
+ * possible, or %NULL in other cases. The caller should not change nor free the
+ * value.
+ *
+ * Since: 0.2.0
+ **/
+gpointer
+grl_data_get_boxed (GrlData *data, GrlKeyID key)
+{
+  const GValue *value = grl_data_get (data, key);
+
+  if (!value || !G_VALUE_HOLDS_BOXED (value)) {
+    return NULL;
+  } else {
+    return g_value_get_boxed (value);
+  }
 }
 
 /**
  * grl_data_remove:
  * @data: data to change
- * @key: (type GObject.ParamSpec): key to remove
+ * @key: (type GrlKeyID): key to remove
  *
  * Removes the first value for @key from @data. If there are other keys related
  * to @key their values will also be removed from @data.
@@ -513,7 +491,7 @@ grl_data_remove (GrlData *data, GrlKeyID key)
 /**
  * grl_data_has_key:
  * @data: data to inspect
- * @key: (type GObject.ParamSpec): key to search
+ * @key: (type GrlKeyID): key to search
  *
  * Checks if @key is in @data.
  *
@@ -535,7 +513,7 @@ grl_data_has_key (GrlData *data, GrlKeyID key)
     return FALSE;
   }
 
-  related_keys = g_hash_table_lookup (data->priv->data, sample_key);
+  related_keys = g_hash_table_lookup (data->priv->data, GRLKEYID_TO_POINTER (sample_key));
   while (related_keys && !found) {
     found = grl_related_keys_has_key (related_keys->data, key);
     related_keys = g_list_next (related_keys);
@@ -550,7 +528,7 @@ grl_data_has_key (GrlData *data, GrlKeyID key)
  *
  * Returns a list with keys contained in @data.
  *
- * Returns: (transfer container) (element-type GObject.ParamSpec): an array with the
+ * Returns: (transfer container) (element-type GrlKeyID): an array with the
  * keys. The content of the list should not be modified or freed. Use
  * g_list_free() when done using the list.
  *
@@ -561,19 +539,20 @@ grl_data_get_keys (GrlData *data)
 {
   GList *allkeys = NULL;
   GList *keylist, *key;
-  GrlPluginRegistry *registry;
+  GrlRegistry *registry;
   const GList *relkeys;
 
   g_return_val_if_fail (GRL_IS_DATA (data), NULL);
 
   keylist = g_hash_table_get_keys (data->priv->data);
-  registry = grl_plugin_registry_get_default ();
+  registry = grl_registry_get_default ();
 
   for (key = keylist; key; key = g_list_next (key)) {
-    relkeys = grl_plugin_registry_lookup_metadata_key_relation (registry,
-                                                                key->data);
+    GrlKeyID key_id = GRLPOINTER_TO_KEYID (key->data);
+    relkeys =
+        grl_registry_lookup_metadata_key_relation (registry, key_id);
     while (relkeys) {
-      if (grl_data_has_key (data, relkeys->data)) {
+      if (grl_data_has_key (data, GRLPOINTER_TO_KEYID (relkeys->data))) {
         allkeys = g_list_prepend (allkeys, relkeys->data);
       }
       relkeys = g_list_next (relkeys);
@@ -585,27 +564,6 @@ grl_data_get_keys (GrlData *data)
   return allkeys;
 }
 
-/**
- * grl_data_key_is_known:
- * @data: data to inspect
- * @key: (type GObject.ParamSpec): key to search
- *
- * Checks if the @key has a first value in @data.
- *
- * Returns: %TRUE if key has a value.
- *
- * Since: 0.1.4
- * Deprecated: 0.1.13: Use grl_data_has_key() instead
- **/
-gboolean
-grl_data_key_is_known (GrlData *data, GrlKeyID key)
-{
-  GRL_WARNING ("grl_data_key_is_known() is deprecated. "
-               "Use instead grl_data_has_key()");
-
-  return grl_data_has_key (data, key);
-}
-
 /**
  * grl_data_add_related_keys:
  * @data: data to change
@@ -638,7 +596,7 @@ grl_data_add_related_keys (GrlData *data,
     return;
   }
 
-  sample_key = get_sample_key (keys->data);
+  sample_key = get_sample_key (GRLPOINTER_TO_KEYID (keys->data));
   g_list_free (keys);
 
   if (!sample_key) {
@@ -646,15 +604,18 @@ grl_data_add_related_keys (GrlData *data,
     return;
   }
 
-  list_relkeys = g_hash_table_lookup (data->priv->data, sample_key);
+  list_relkeys = g_hash_table_lookup (data->priv->data,
+                                      GRLKEYID_TO_POINTER (sample_key));
   list_relkeys = g_list_append (list_relkeys, relkeys);
-  g_hash_table_insert (data->priv->data, sample_key, list_relkeys);
+  g_hash_table_insert (data->priv->data,
+                       GRLKEYID_TO_POINTER (sample_key),
+                       list_relkeys);
 }
 
 /**
  * grl_data_add_string:
  * @data: data to append
- * @key: (type GObject.ParamSpec): key to append
+ * @key: (type GrlKeyID): key to append
  * @strvalue: the new value
  *
  * Appends a new string value for @key in @data.
@@ -678,7 +639,7 @@ grl_data_add_string (GrlData *data,
 /**
  * grl_data_add_int:
  * @data: data to append
- * @key: (type GObject.ParamSpec): key to append
+ * @key: (type GrlKeyID): key to append
  * @intvalue: the new value
  *
  * Appends a new int value for @key in @data.
@@ -700,7 +661,7 @@ grl_data_add_int (GrlData *data,
 /**
  * grl_data_add_float:
  * @data: data to append
- * @key: (type GObject.ParamSpec): key to append
+ * @key: (type GrlKeyID): key to append
  * @floatvalue: the new value
  *
  * Appends a new float value for @key in @data.
@@ -722,7 +683,7 @@ grl_data_add_float (GrlData *data,
 /**
  * grl_data_add_binary:
  * @data: data to append
- * @key: (type GObject.ParamSpec): key to append
+ * @key: (type GrlKeyID): key to append
  * @buf: the buffer containing the new value
  * @size: size of buffer
  *
@@ -747,6 +708,30 @@ grl_data_add_binary (GrlData *data,
   grl_data_add_related_keys (data, relkeys);
 }
 
+/**
+ * grl_data_add_boxed:
+ * @data: data to append
+ * @key: (type GrlKeyID): key to append
+ * @boxed: the new value
+ *
+ * Appends a new boxed value for @key in @data.
+ *
+ * Since: 0.2.0
+ **/
+void
+grl_data_add_boxed (GrlData *data,
+                    GrlKeyID key,
+                    gconstpointer boxed)
+{
+  GrlRelatedKeys *relkeys;
+
+  g_return_if_fail (boxed != NULL);
+
+  relkeys = grl_related_keys_new ();
+  grl_related_keys_set_boxed (relkeys, key, boxed);
+  grl_data_add_related_keys (data, relkeys);
+}
+
 /**
  * grl_data_length:
  * @data: a data
@@ -781,7 +766,8 @@ grl_data_length (GrlData *data,
     return 0;
   }
 
-  return g_list_length (g_hash_table_lookup (data->priv->data, sample_key));
+  return g_list_length (g_hash_table_lookup (data->priv->data,
+                                             GRLKEYID_TO_POINTER (sample_key)));
 }
 
 /**
@@ -817,7 +803,8 @@ grl_data_get_related_keys (GrlData *data,
     return NULL;
   }
 
-  relkeys_list = g_hash_table_lookup (data->priv->data, sample_key);
+  relkeys_list = g_hash_table_lookup (data->priv->data,
+                                      GRLKEYID_TO_POINTER (sample_key));
   relkeys = g_list_nth_data (relkeys_list, index);
 
   if (!relkeys) {
@@ -828,29 +815,6 @@ grl_data_get_related_keys (GrlData *data,
   return relkeys;
 }
 
-/**
- * grl_data_get_all_single_related_keys:
- * @data: a data
- * @key: a metadata key
- *
- * Returns all non-%NULL values for @key from @data. This ignores related keys.
- *
- * Returns: (element-type GObject.Value) (transfer container): a #GList with
- * values. Do not change or free the values. Free the list with #g_list_free.
- *
- * Since: 0.1.10
- * Deprecated: 0.1.13: Use grl_data_get_single_values_for_key() instead
- **/
-GList *
-grl_data_get_all_single_related_keys (GrlData *data,
-                                      GrlKeyID key)
-{
-  GRL_WARNING ("grl_data_get_all_single_related_keys() is deprecated. "
-               "Use instead grl_data_get_single_values_for_key()");
-
-  return grl_data_get_single_values_for_key (data, key);
-}
-
 /**
  * grl_data_get_single_values_for_key:
  * @data: a data
@@ -880,7 +844,8 @@ grl_data_get_single_values_for_key (GrlData *data,
     return NULL;
   }
 
-  related_keys = g_hash_table_lookup (data->priv->data, sample_key);
+  related_keys = g_hash_table_lookup (data->priv->data,
+                                      GRLKEYID_TO_POINTER (sample_key));
   while (related_keys) {
     v = grl_related_keys_get (related_keys->data, key);
     if (v) {
@@ -892,30 +857,6 @@ grl_data_get_single_values_for_key (GrlData *data,
   return g_list_reverse (values);
 }
 
-/**
- * grl_data_get_all_single_related_keys_string:
- * @data: a data
- * @key: a metadata key
- *
- * Returns all non-%NULL values for @key from @data. @key must have been
- * registered as a string-type key. This ignores related keys.
- *
- * Returns: (element-type utf8) (transfer container): a #GList with values. Do
- * not change or free the strings. Free the list with #g_list_free.
- *
- * Since: 0.1.10
- * Deprecated: 0.1.13: Use grl_data_get_single_values_for_key_string() instead
- **/
-GList *
-grl_data_get_all_single_related_keys_string (GrlData *data,
-                                             GrlKeyID key)
-{
-  GRL_WARNING ("grl_data_get_all_single_related_keys_string() is deprecated. "
-               "Use instead grl_data_get_single_values_for_key_string()");
-
-  return grl_data_get_single_values_for_key_string (data, key);
-}
-
 /**
  * grl_data_get_single_values_for_key_string:
  * @data: a data
@@ -989,7 +930,8 @@ grl_data_remove_nth (GrlData *data,
     return;
   }
 
-  relkeys_list = g_hash_table_lookup (data->priv->data, sample_key);
+  relkeys_list = g_hash_table_lookup (data->priv->data,
+                                      GRLKEYID_TO_POINTER (sample_key));
   relkeys_element = g_list_nth (relkeys_list, index);
   if (!relkeys_element) {
     GRL_WARNING ("%s: index %u out of range", __FUNCTION__, index);
@@ -998,7 +940,9 @@ grl_data_remove_nth (GrlData *data,
 
   g_object_unref (relkeys_element->data);
   relkeys_list = g_list_delete_link (relkeys_list, relkeys_element);
-  g_hash_table_insert (data->priv->data, sample_key, relkeys_list);
+  g_hash_table_insert (data->priv->data,
+                       GRLKEYID_TO_POINTER (sample_key),
+                       relkeys_list);
 }
 
 /**
@@ -1034,13 +978,14 @@ grl_data_set_related_keys (GrlData *data,
     return;
   }
 
-  sample_key = get_sample_key (keys->data);
+  sample_key = get_sample_key (GRLPOINTER_TO_KEYID (keys->data));
   g_list_free (keys);
   if (!sample_key) {
     return;
   }
 
-  relkeys_list = g_hash_table_lookup (data->priv->data, sample_key);
+  relkeys_list = g_hash_table_lookup (data->priv->data,
+                                      GRLKEYID_TO_POINTER (sample_key));
   relkeys_element = g_list_nth (relkeys_list, index);
   if (!relkeys_element) {
     GRL_WARNING ("%s: index %u out of range", __FUNCTION__, index);
@@ -1092,41 +1037,3 @@ grl_data_dup (GrlData *data)
 
   return dup_data;
 }
-
-/**
- * grl_data_set_overwrite:
- * @data: data to change
- * @overwrite: if data can be overwritten
- *
- * This controls if #grl_data_set will overwrite the current value of a property
- * with the new one.
- *
- * Set it to %TRUE so old values are overwritten, or %FALSE in other case
- * (default is %FALSE).
- *
- * Since: 0.1.4
- * Deprecated: 0.1.13
- **/
-void
-grl_data_set_overwrite (GrlData *data, gboolean overwrite)
-{
-  GRL_WARNING ("\"overwrite\" property is deprecated");
-}
-
-/**
- * grl_data_get_overwrite:
- * @data: data to inspect
- *
- * Checks if old values are replaced when calling #grl_data_set.
- *
- * Returns: %TRUE if values will be overwritten.
- *
- * Since: 0.1.4
- * Deprecated: 0.1.13
- **/
-gboolean
-grl_data_get_overwrite (GrlData *data)
-{
-  GRL_WARNING ("\"overwrite\" property is deprecated");
-  return TRUE;
-}
