@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 Igalia S.L.
+ * Copyright (C) 2010, 2011 Igalia S.L.
  *
  * Contact: Iago Toral Quiroga <itoral@igalia.com>
  *
@@ -141,6 +141,14 @@ _grl_log_domain_free_internal (GrlLogDomain *domain)
   g_slice_free (GrlLogDomain, domain);
 }
 
+/**
+ * grl_log_domain_free:
+ * @domain: a #GrlLogDomain
+ *
+ * Releases @domain.
+ *
+ * Since: 0.1.7
+ **/
 void
 grl_log_domain_free (GrlLogDomain *domain)
 {
@@ -296,6 +304,18 @@ grl_log_valist (GrlLogDomain *domain,
   g_free (message);
 }
 
+/**
+ * grl_log:
+ * @domain: a domain
+ * @level: log level
+ * @strloc: string, usually line of code where function is invoked
+ * @format: log message
+ * @...: parameters to insert in the log message
+ *
+ * Send a log message.
+ *
+ * Since: 0.1.7
+ **/
 void
 grl_log (GrlLogDomain *domain,
          GrlLogLevel   level,
@@ -326,11 +346,10 @@ _grl_log_init_core_domains (void)
   DOMAIN_INIT (config_log_domain, "config");
   DOMAIN_INIT (data_log_domain, "data");
   DOMAIN_INIT (media_log_domain, "media");
-  DOMAIN_INIT (media_plugin_log_domain, "media-plugin");
-  DOMAIN_INIT (media_source_log_domain, "media-source");
-  DOMAIN_INIT (metadata_source_log_domain, "metadata-source");
+  DOMAIN_INIT (plugin_log_domain, "plugin");
+  DOMAIN_INIT (source_log_domain, "source");
   DOMAIN_INIT (multiple_log_domain, "multiple");
-  DOMAIN_INIT (plugin_registry_log_domain, "plugin-registry");
+  DOMAIN_INIT (registry_log_domain, "registry");
 
   /* Retrieve the GRL_DEBUG environment variable, initialize core domains from
    * it if applicable and keep it for grl_log_domain_new(). Plugins are using
@@ -371,11 +390,10 @@ _grl_log_free_core_domains (void)
   DOMAIN_FREE (log_log_domain);
   DOMAIN_FREE (config_log_domain);
   DOMAIN_FREE (media_log_domain);
-  DOMAIN_FREE (media_plugin_log_domain);
-  DOMAIN_FREE (media_source_log_domain);
-  DOMAIN_FREE (metadata_source_log_domain);
+  DOMAIN_FREE (plugin_log_domain);
+  DOMAIN_FREE (source_log_domain);
   DOMAIN_FREE (multiple_log_domain);
-  DOMAIN_FREE (plugin_registry_log_domain);
+  DOMAIN_FREE (registry_log_domain);
 
   g_strfreev (grl_log_env);
 }
