@@ -32,7 +32,7 @@ static gint delay = 0;
 static GMainLoop *mainloop = NULL;
 static gchar **introspect_sources = NULL;
 static gchar *conffile = NULL;
-static GrlPluginRegistry *registry = NULL;
+static GrlRegistry *registry = NULL;
 static gboolean version;
 
 static GOptionEntry entries[] = {
@@ -61,16 +61,19 @@ list_all_sources ()
   GList *sources = NULL;
   GList *sources_iter;
 
-  sources = grl_plugin_registry_get_sources (registry, FALSE);
+  sources = grl_registry_get_sources (registry, FALSE);
 
   for (sources_iter = sources; sources_iter;
       sources_iter = g_list_next (sources_iter)) {
-    GrlMediaPlugin *source;
+    GrlSource *source;
+    GrlPlugin *plugin;
+
+    source = GRL_SOURCE (sources_iter->data);
+    plugin = grl_source_get_plugin (source);
 
-    source = GRL_MEDIA_PLUGIN (sources_iter->data);
     g_print ("%s:  %s\n",
-             grl_media_plugin_get_id (source),
-             grl_metadata_source_get_id (GRL_METADATA_SOURCE (source)));
+             grl_plugin_get_id (plugin),
+             grl_source_get_id (source));
   }
   g_list_free (sources);
 }
@@ -79,7 +82,8 @@ static void
 print_keys (const GList *keys)
 {
   while (keys) {
-    g_print ("%s", GRL_METADATA_KEY_GET_NAME (keys->data));
+    g_print ("%s",
+             GRL_METADATA_KEY_GET_NAME (GRLPOINTER_TO_KEYID (keys->data)));
     keys = g_list_next (keys);
     if (keys) {
       g_print (", ");
@@ -98,57 +102,53 @@ print_version()
 static void
 introspect_source (const gchar *source_id)
 {
-  GrlMediaPlugin *source;
+  GrlPlugin *plugin;
+  GrlSource *source;
   GrlSupportedOps supported_ops;
   const gchar *value;
   gchar *key;
   GList *info_keys;
   GList *info_key;
 
-  source = grl_plugin_registry_lookup_source (registry, source_id);
+  source = grl_registry_lookup_source (registry, source_id);
 
   if (source) {
-    g_print ("Plugin Details:\n");
-    g_print ("  %-20s %s\n", "Identifier:", grl_media_plugin_get_id (source));
-    g_print ("  %-20s %s\n", "Filename:",
-             grl_media_plugin_get_filename (source));
-    g_print ("  %-20s %d\n", "Rank:", grl_media_plugin_get_rank (source));
+    plugin = grl_source_get_plugin (source);
 
-    info_keys = grl_media_plugin_get_info_keys (source);
-    for (info_key = info_keys; info_key; info_key = g_list_next (info_key)) {
-      key = g_strdup_printf ("%s:", (gchar *) info_key->data);
-      key[0] = g_ascii_toupper (key[0]);
-      value = grl_media_plugin_get_info (source, info_key->data);
-      g_print ("  %-20s %s\n", key, value);
-      g_free (key);
+    if (plugin) {
+      g_print ("Plugin Details:\n");
+      g_print ("  %-20s %s\n", "Identifier:", grl_plugin_get_id (plugin));
+      g_print ("  %-20s %s\n", "Filename:",
+               grl_plugin_get_filename (plugin));
+
+      info_keys = grl_plugin_get_info_keys (plugin);
+      for (info_key = info_keys; info_key; info_key = g_list_next (info_key)) {
+        key = g_strdup_printf ("%s:", (gchar *) info_key->data);
+        key[0] = g_ascii_toupper (key[0]);
+        value = grl_plugin_get_info (plugin, info_key->data);
+        g_print ("  %-20s %s\n", key, value);
+        g_free (key);
+      }
+      g_list_free (info_keys);
+      g_print ("\n");
     }
-    g_list_free (info_keys);
-    g_print ("\n");
 
     g_print ("Source Details:\n");
     g_print ("  %-20s %s\n", "Identifier:",
-             grl_metadata_source_get_id (GRL_METADATA_SOURCE (source)));
-    g_print ("  %-20s %s\n", "Type:",
-             GRL_IS_MEDIA_SOURCE (source)? "Media Provider": "Metadata Provider");
+             grl_source_get_id (source));
     g_print ("  %-20s %s\n", "Name:",
-             grl_metadata_source_get_name (GRL_METADATA_SOURCE (source)));
+             grl_source_get_name (source));
     g_print ("  %-20s %s\n", "Description:",
-             grl_metadata_source_get_description (GRL_METADATA_SOURCE (source)));
+             grl_source_get_description (source));
     g_print ("\n");
 
     /* Print supported operations */
     supported_ops =
-      grl_metadata_source_supported_operations (GRL_METADATA_SOURCE (source));
+      grl_source_supported_operations (source);
     g_print ("Supported operations:\n");
     if (supported_ops & GRL_OP_RESOLVE) {
       g_print ("  grl_metadata_source_resolve():\tResolve Metadata\n");
     }
-    if (supported_ops & GRL_OP_SET_METADATA) {
-      g_print ("  grl_metadata_source_set_metadata():\tSet Metadata\n");
-    }
-    if (supported_ops & GRL_OP_METADATA) {
-      g_print ("  grl_media_source_metadata():\t\tRetrieve Metadata\n");
-    }
     if (supported_ops & GRL_OP_BROWSE) {
       g_print ("  grl_media_source_browse():\t\tBrowse\n");
     }
@@ -164,6 +164,9 @@ introspect_source (const gchar *source_id)
     if (supported_ops & GRL_OP_STORE_PARENT) {
       g_print ("  grl_media_source_store():\t\tAdd New Media\n");
     }
+    if (supported_ops & GRL_OP_STORE_METADATA) {
+      g_print ("  grl_metadata_source_store_metadata():\tStore Metadata\n");
+    }
     if (supported_ops & GRL_OP_REMOVE) {
       g_print ("  grl_media_source_remove():\t\tRemove Media\n");
     }
@@ -179,10 +182,10 @@ introspect_source (const gchar *source_id)
     /* Print supported keys */
     g_print ("Supported keys:\n");
     g_print ("  Readable Keys:\t");
-    print_keys (grl_metadata_source_supported_keys (GRL_METADATA_SOURCE (source)));
+    print_keys (grl_source_supported_keys (source));
     g_print ("\n");
     g_print ("  Writable Keys:\t");
-    print_keys (grl_metadata_source_writable_keys (GRL_METADATA_SOURCE (source)));
+    print_keys (grl_source_writable_keys (source));
     g_print ("\n");
     g_print ("\n");
   } else {
@@ -236,9 +239,9 @@ main (int argc, char *argv[])
 
   GRL_LOG_DOMAIN_INIT (grl_inspect_log_domain, "grl-inspect");
 
-  registry = grl_plugin_registry_get_default ();
+  registry = grl_registry_get_default ();
   if (conffile) {
-    grl_plugin_registry_add_config_from_file (registry, conffile, &error);
+    grl_registry_add_config_from_file (registry, conffile, &error);
     if (error) {
       GRL_WARNING ("Unable to load configuration: %s", error->message);
       g_error_free (error);
@@ -247,7 +250,7 @@ main (int argc, char *argv[])
 
   mainloop = g_main_loop_new (NULL, FALSE);
 
-  grl_plugin_registry_load_all (registry, NULL);
+  grl_registry_load_all_plugins (registry, NULL);
 
   if (delay > 0) {
     g_timeout_add_seconds ((guint) delay, run, NULL);
